import { resolve } from 'node:path';
import { readdirSync, existsSync, lstatSync, mkdirSync, createReadStream, createWriteStream } from 'node:fs';
import { fileURLToPath } from 'node:url';
import inquirer from 'inquirer';

const getCurDir = url => {
    return fileURLToPath(url);
};
/**
 * 判断一个url是不是目录
 * @param {*} dir 检测的url
 * @returns
 */
const isDir = path => {
    try {
        var stat = lstatSync(path);
        const flag = stat.isDirectory();
        return flag;
    } catch (e) {
        return false;
    }
};

const mkdir = path => {
    try {
        mkdirSync(path);
        return true;
    } catch (error) {
        return false;
    }
};

const copyTo = (src, dest) => {
    createReadStream(src).pipe(createWriteStream(dest));
};

const copyDir = (src, dest) => {
    if (isDir(src) && isDir(dest)) {
        const dirList = readdirSync(src);
        dirList.forEach(f => {
            const tmpSrc = resolve(src, f);
            const tmpDest = resolve(dest, f);
            if (isDir(tmpSrc)) {
                if (existsSync(tmpDest)) {
                    copyDir(tmpSrc, tmpDest);
                } else {
                    const flag = mkdir(tmpDest);
                    if (!flag) {
                        throw new Error(`目录创建失败：创建失败的目录路径-${tmpDest}`);
                    } else {
                        copyDir(tmpSrc, tmpDest);
                    }
                }
            } else {
                copyTo(tmpSrc, tmpDest);
            }
        });
    } else {
        throw new Error(`copyDir的两个参数都必须是文件夹类型`);
    }
};

const root = getCurDir(import.meta.url).replace("bin\\index.mjs", "");

inquirer
    .prompt([
        {
            type: "input", //type： input, number, confirm, list, checkbox ...
            name: "name", // key 名
            message: "project name", // 提示信息
            default: "project-name", // 默认值
        },
        {
            type: "list", //type： input, number, confirm, list, checkbox ...
            name: "vue-version", // key 名
            message: "", // 提示信息
            choices: ["vue2", "vue3", "vue3-webpack"],
            default: "vue3-webpack", // 默认值
        },
    ])
    .then(answers => {
        // 打印互用输入结果
        const map = {
            vue2: "vue2",
            vue3: "vue",
            "vue3-webpack": "vue-webpack",
        };

        const version = answers["vue-version"];
        answers.name;

        const dist = resolve(root, "dist");
        const src = resolve(root, `template\\${map[version]}`);
        console.log(answers, src);
        copyDir(src, dist);
    });
